# Generated by Django 5.0.9 on 2024-10-03 10:20

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('sop_phone', '0002_phonemaintainer_latitude_phonemaintainer_longitude_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='phonemaintainer',
            name='shipping_address',
            field=models.CharField(blank=True, max_length=200),
        ),
    ]



# Generated by Django 5.0.9 on 2024-10-03 08:51

import timezone_field.fields
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('sop_phone', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='phonemaintainer',
            name='latitude',
            field=models.DecimalField(blank=True, decimal_places=6, max_digits=8, null=True),
        ),
        migrations.AddField(
            model_name='phonemaintainer',
            name='longitude',
            field=models.DecimalField(blank=True, decimal_places=6, max_digits=9, null=True),
        ),
        migrations.AddField(
            model_name='phonemaintainer',
            name='physical_address',
            field=models.CharField(blank=True, max_length=200),
        ),
        migrations.AddField(
            model_name='phonemaintainer',
            name='time_zone',
            field=timezone_field.fields.TimeZoneField(blank=True),
        ),
    ]



from django.db import models
from django.urls import reverse
from django.db.models import Transform
from django.core.exceptions import ValidationError
from django.utils.translation import gettext_lazy as _

from timezone_field import TimeZoneField

from netbox.models import NetBoxModel, PrimaryModel
from netbox.models.features import ContactsMixin
from utilities.choices import ChoiceSet
from circuits.models import Provider
from dcim.models import Site

from scripts.sop_utils import CheckResultList, ValidatorCheckResultLogger

from .validators import PhoneValidator, PhoneMaintainerValidator



__all__ = (
    'PhoneDID',
    'LogValue',
    'PhoneInfo',
    'FloorValue',
    'PhoneDelivery',
    'AbsoluteValue',
    'PhoneMaintainer',
    'PhoneBoolChoices',
    'PhoneMaintainerStatusChoice',
    'PhoneDeliveryStatusChoices',
)


class PhoneMaintainerStatusChoice(ChoiceSet):

    CHOICES = (
        ('active', _('Active'), 'green'),
        ('retired', _('Retired'), 'red'),
        ('unknown', _('Unknown'), 'gray'),
    )


class PhoneDeliveryStatusChoices(ChoiceSet):

    CHOICES = (
        ('active', _('Active'), 'green'),
        ('planned', _('Planned'), 'cyan'),
        ('staging', _('Staging'), 'blue'),
        ('retired', _('Retired'), 'red'),
        ('unknown', _('Unknown'), 'gray'),
    )


class PhoneBoolChoices(ChoiceSet):

    CHOICES = (
        ('unknown', _('Unknown'), 'gray'),
        ('true', _('True'), 'green'),
        ('false', _('False'), 'red'),
    )


class AbsoluteValue(Transform):
    lookup_name = "abs"
    function = "ABS"
    bilateral = True

class FloorValue(Transform):
    lookup_name = "floor"
    function = "FLOOR"
    bilateral = True

class LogValue(Transform):
    lookup_name = "log"
    function = "LOG"
    bilateral = True


models.IntegerField.register_lookup(AbsoluteValue)
models.IntegerField.register_lookup(FloorValue)
models.IntegerField.register_lookup(LogValue)


class BiAbsoluteValue(Transform):
    lookup_name = "biabs"
    function = "ABS"
    bilateral = True

class BiFloorValue(Transform):
    lookup_name = "bifloor"
    function = "FLOOR"
    bilateral = True

class BiLogValue(Transform):
    lookup_name = "bilog"
    function = "LOG"
    bilateral = True

models.IntegerField.register_lookup(BiAbsoluteValue)
models.IntegerField.register_lookup(BiFloorValue)
models.IntegerField.register_lookup(BiLogValue)

class PhoneMaintainer(PrimaryModel, ContactsMixin):
    name = models.CharField(
        verbose_name=_('Maintainer'),
        unique=True,
    )
    slug = models.SlugField(
        verbose_name=_('slug'),
        max_length=100,
        unique=True,
        blank=True,
    )
    status = models.CharField(
        max_length=30,
        choices=PhoneMaintainerStatusChoice,
        default="Unknown",
        verbose_name=_('Status')
    )
    physical_address = models.CharField(
        verbose_name=_('Physical address'),
        max_length=200,
        null=True,
        blank=True,
        help_text=_('Physical location of the maintainer')
    )
    shipping_address = models.CharField(
        verbose_name=_('Shipping address'),
        max_length=200,
        null=True,
        blank=True,
        help_text=_('If different from the physical address')
    )
    time_zone = TimeZoneField(
        null=True,
        blank=True
    )
    latitude = models.DecimalField(
        verbose_name=_('Latitude'),
        max_digits=8,
        decimal_places=6,
        blank=True,
        null=True,
        help_text=_('GPS coordinate in decimal format (xx.yyyyyy)')
    )
    longitude = models.DecimalField(
        verbose_name=_('Longitude'),
        max_digits=9,
        decimal_places=6,
        blank=True,
        null=True,
        help_text=_('GPS coordinate in decimal format (xx.yyyyyy)')
    )

    def __str__(self) -> str:
        return f'{self.name}'

    def get_absolute_url(self) -> str:
        return reverse('plugins:sop_phone:phonemaintainer_detail', args=[self.pk])

    def get_status_color(self) -> str:
        return PhoneDeliveryStatusChoices.colors.get(self.status)

    class Meta(NetBoxModel.Meta):
        verbose_name = _('Phone Maintainer')
        verbose_name_plural = _('Phone Maintainers')
        constraints = (
            models.UniqueConstraint(
                fields=('name',),
                name='%(app_label)s_%(class)s_unique_name',
                violation_error_message=_("Maintainer name must be unique.")
            ),
            models.UniqueConstraint(
                fields=('slug',),
                name='%(app_label)s_%(class)s_unique_slug',
                violation_error_message=_("Maintainer slug must be unique.")
            )
        )

    def fail(self, message, field=None):
        if field is not None:
            raise ValidationError({field: message})
        raise ValidationError(message)

    def clean(self):
        super().clean()

        failprefix = ''
        # NOK A REVOIR
        # crl = CheckResultList()
        #PhoneMaintainerValidator.check_time_zone(self.status, self.time_zone, crl)
        #PhoneMaintainerValidator.check_address(self.status, self.latitude, self.longitude, self.physical_address, crl)
        #crl.dump_to(ValidatorCheckResultLogger(self, failprefix))


class PhoneInfo(NetBoxModel):
    site = models.OneToOneField(
        Site,
        on_delete=models.CASCADE,
        verbose_name=_('Site'),
    )
    maintainer = models.ForeignKey(
        PhoneMaintainer,
        on_delete=models.SET_NULL,
        null=True,
        blank=True,
        verbose_name=_('Maintainer'),
    )

    def __str__(self) -> str:
        return f'{self.site} phone maintainer'

    def get_absolute_url(self) -> str:
        return reverse('plugins:sop_phone:phoneinfo_detail', args=[self.pk])

    class Meta(NetBoxModel.Meta):
        verbose_name = _('Information')
        verbose_name_plural = _('Informations')
        constraints = [
            models.UniqueConstraint(
                fields=['site',],
                name='%(app_label)s_%(class)s_unique_site',
                violation_error_message=_("Site must be unique.")
            )
        ]


class PhoneDelivery(NetBoxModel):
    delivery = models.CharField(
        verbose_name=_('Delivery'),
        null=True,
        blank=True,
    )
    provider = models.ForeignKey(
        Provider,
        on_delete=models.RESTRICT,
        verbose_name=_('Provider'),
        null=True,
        blank=True,
    )
    site = models.ForeignKey(
        Site,
        on_delete=models.RESTRICT,
        verbose_name=_('Site'),
    )
    channel_count = models.PositiveBigIntegerField(
        verbose_name=_('Channel Count'),
        null=True,
        blank=True,
    )
    status = models.CharField(
        max_length=30,
        choices=PhoneDeliveryStatusChoices,
        verbose_name=_('Status'),
    )
    ndi = models.PositiveBigIntegerField(
        verbose_name=_('NDI'),
        null=True,
        blank=True,
    )
    dto = models.PositiveBigIntegerField(
        verbose_name=_('DTO'),
        null=True,
        blank=True,
    )
    description = models.CharField(
        verbose_name=_('description'),
        max_length=200,
        blank=True
    )
    comments = models.TextField(
        verbose_name=_('comments'),
        blank=True
    )

    def get_absolute_url(self) -> str:
        return reverse('plugins:sop_phone:phonedelivery_detail', args=[self.pk])

    def get_status_color(self) -> str:
        return PhoneDeliveryStatusChoices.colors.get(self.status)

    def __str__(self) -> str:
        delivery:str = self.delivery if hasattr(self, 'delivery') else 'Unknown delivery'
        provider:str = self.provider if hasattr(self, 'provider') else 'Unknown provider'
        return f'{delivery} / {provider}'

    def clean(self):
        super().clean()
        if hasattr(self, 'delivery') and self.delivery and self.site:
            if PhoneDelivery.objects.filter(site=self.site, delivery=self.delivery).exists():
                raise ValidationError({
                    'delivery': _(f'A "{self.delivery}" delivery method already exists for this site.')
                })
        if self.ndi:
            PhoneValidator.check_number('ndi', self.ndi)
            if PhoneDelivery.objects.filter(ndi=self.ndi).exists():
                raise ValidationError({
                    'ndi': _(f'The NDI {self.ndi} already exists on another delivery.')
                })


    class Meta(NetBoxModel.Meta):
        verbose_name = _('Phone Delivery')
        verbose_name_plural = _('Phone Deliveries')
        constraints = (
            models.UniqueConstraint(
                fields=('ndi',),
                name='%(app_label)s_%(class)s_unique_ndi',
                violation_error_message=_("NDI must be unique.")
            ),
            models.UniqueConstraint(
                fields=('delivery', 'site'),
                name='%(app_label)s_%(class)s_unique_delivery_method_site',
                violation_error_message=_("Delivery method must be unique in a site.")
            )
        )


class PhoneDID(NetBoxModel):
    delivery = models.ForeignKey(
        PhoneDelivery,
        on_delete=models.SET_NULL,
        verbose_name=_('Delivery'),
        null=True,
        blank=True,
    )
    site = models.ForeignKey(
        Site,
        on_delete=models.CASCADE,
        verbose_name=_('Site'),
        null=False,
        blank=True,
    )
    start = models.PositiveBigIntegerField(
        unique=False,
        verbose_name=_('Start number'),
        null=False,
        blank=True,
    )
    end = models.PositiveBigIntegerField(
        unique=False,
        verbose_name=_('End number'),
        null=False,
        blank=True,
    )

    def __str__(self) -> str:
        return f'"{self.start} >> {self.end}"'

    def get_absolute_url(self) -> str:
        return reverse('plugins:sop_phone:phonedid_detail', args=[self.pk])

    def clean(self):
        super().clean()

        PhoneValidator.check_delivery(self.delivery, self.site)

        PhoneValidator.check_number('start', self.start)
        if self.end is None:
            self.end=self.start
        else :
            PhoneValidator.check_number('end', self.end)
            PhoneValidator.check_start_end(self.start, self.end)  

        '''
        check if self.end or self.start overlaps an existing DID range
        '''
        lnum=len(str(self.start))
        for rng in PhoneDID.objects.all():
            #only compare if numbers are comaprable (have the same length)
            if len(str(rng.start))==lnum:
                # check if number overlap
                if self.start <= rng.end and rng.start <= self.end:
                    raise ValidationError({
                        'start': _(f'This range {self.start} -> {self.end} overlaps range {rng.start} -> {rng.end}.')
                    })

    def save(self, *args, **kwargs):
        if not self.end or self.end == 0:
            self.end = self.start
        super().save(*args, **kwargs)

    class Meta(NetBoxModel.Meta):
        ordering = ('start',)
        verbose_name = _('DID Range')
        verbose_name_plural = _('DIDs')

        constraints = (
            models.UniqueConstraint(
                fields=('start',),
                name='%(app_label)s_%(class)s_unique_start_number',
                violation_error_message=_("Start number must be unique.")
            ),
            models.UniqueConstraint(
                fields=('end',),
                name='%(app_label)s_%(class)s_unique_end_number',
                violation_error_message=_("End number must be unique.")
            ),
            models.CheckConstraint(
                check=models.Q(end__gte=models.F('start')) & \
                    models.Q(start__biabs__bilog__bifloor=models.F("end")),
                name='%(app_label)s_%(class)s_end_greater_than_start',
                violation_error_message=_("End number must be greater than or equal to start number.")
            )
        )



from django import forms
from django.utils.translation import gettext_lazy as _

from dcim.models import Site
from circuits.models import Provider
from utilities.forms.fields import SlugField
from utilities.forms.fields import DynamicModelChoiceField, CommentField
from netbox.forms import NetBoxModelFilterSetForm, NetBoxModelForm, NetBoxModelBulkEditForm

from ..models import *


__all__ = (
    'PhoneDeliveryForm',
    'PhoneDeliveryFilterForm',
    'PhoneDeliveryBulkEditForm',
)


class PhoneDeliveryFilterForm(NetBoxModelFilterSetForm):
    model = PhoneDelivery

    site_id = DynamicModelChoiceField(
        queryset=Site.objects.all(),
        required=False,
        label=_('Site')
    )
    delivery_id = forms.CharField(
        label=_('Delivery Method'),
        required=False
    )
    provider = forms.ModelChoiceField(
        required=False,
        queryset=Provider.objects.all(),
        label=_('Provider')
    )
    status = forms.ChoiceField(
        choices=PhoneDeliveryStatusChoices,
        required=False,
        label=_('Status')
    )


class PhoneDeliveryBulkEditForm(NetBoxModelBulkEditForm):
    model = PhoneDelivery

    site = forms.ModelChoiceField(
        queryset=Site.objects.all(),
        required=False,
        label=_('Site')
    )
    delivery = forms.CharField(
        required=False,
        label=_('Delivery Method'),
        help_text=_('SIP TRUNK, T0, T2, ...')
    )
    provider = forms.ModelChoiceField(
        queryset=Provider.objects.all(),
        required=False,
        label=_('Provider')
    )
    status = forms.ChoiceField(
        choices=PhoneDeliveryStatusChoices,
        required=False,
        label=_('Status')
    )

    class Meta:
        fields = ('site', 'delivery', 'provider', 'status', )

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        if 'add_tags' in self.fields:
            del self.fields['add_tags']
        if 'remove_tags' in self.fields:
            del self.fields['remove_tags']


class PhoneDeliveryForm(NetBoxModelForm):
    '''
    creates a form for a Phone Delivery object
    '''
    site = forms.ModelChoiceField(
        required=True,
        queryset=Site.objects.all(),
        label=_('Site')
    )
    delivery = forms.CharField(
        required=True,
        label=_('Delivery Method'),
        help_text=_('SIP TRUNK, T0, T2, ...')
    )
    provider = forms.ModelChoiceField(
        required=True,
        queryset=Provider.objects.all(),
        label=_('Provider')
    )
    channel_count = forms.IntegerField(
        required=False,
        label=_('Channel Count'),
        help_text=_('G.711 cidec - 96kbps reserved bandwidth per channel')
    )
    status = forms.ChoiceField(
        choices=PhoneDeliveryStatusChoices,
        required=True,
        label=_('Status'),
    )
    ndi = forms.IntegerField(
        required=False,
        label=_('MBN / NDI'),
        help_text=_("Main Billing Number / Numéro de Désignation d'Installation - E164 format / NUMBER ONLY")
    )
    dto = forms.IntegerField(
        required=False,
        label=_('DTO'),
        help_text=_('E164 format / NUMBER ONLY')
    )
    comments = CommentField()

    class Meta:
        model = PhoneDelivery
        fields = ('site', 'delivery', 'provider', 'channel_count', 'status', 'ndi', 'dto', 'description', 'comments')

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        if 'tags' in self.fields:
            del self.fields['tags']



# Generated by Django 5.0.9 on 2024-10-03 14:11

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('circuits', '0044_circuit_groups'),
        ('dcim', '0190_nested_modules'),
        ('extras', '0121_customfield_related_object_filter'),
        ('sop_phone', '0003_phonemaintainer_shipping_address'),
    ]

    operations = [
        migrations.RemoveConstraint(
            model_name='phonedid',
            name='sop_phone_phonedid_end_greater_than_start',
        ),
        migrations.AlterField(
            model_name='phonedelivery',
            name='provider',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.RESTRICT, to='circuits.provider'),
        ),
        migrations.AlterField(
            model_name='phonedelivery',
            name='site',
            field=models.ForeignKey(on_delete=django.db.models.deletion.RESTRICT, to='dcim.site'),
        ),
        migrations.AlterField(
            model_name='phonemaintainer',
            name='name',
            field=models.CharField(unique=True),
        ),
        migrations.AddConstraint(
            model_name='phonedid',
            constraint=models.CheckConstraint(check=models.Q(('end__gte', models.F('start')), ('start__biabs__bilog__bifloor', models.F('end'))), name='sop_phone_phonedid_end_greater_than_start', violation_error_message='End number must be greater than or equal to start number.'),
        ),
    ]



import re
import phonenumbers

from django.utils.translation import gettext_lazy as _
from django.core.exceptions import ValidationError

from extras.choices import LogLevelChoices

from scripts.sop_utils import CheckResult


__all__ = (
    'PhoneValidator',
    'PhoneMaintainerValidator',
    'number_quicksearch',
)


class PhoneValidator:

    @staticmethod
    def check_site(site) -> None:
        if site is None:
            raise ValidationError({
                'site': _("Site must be set.")
            })

    @staticmethod
    def check_delivery(delivery, site) -> None:
        if delivery and site and delivery.site != site:
            raise ValidationError({
                'delivery': _("Delivery must be set to the same site as the DID.")
            })

    @staticmethod
    def check_number(where:str, number:int) -> None:
        if number is None or number <= 0 :
            raise ValidationError({
                f'{where}': _("Number must be set in E164 format.")
            })
        if not phonenumbers.parse(f'+{number}'):
            raise ValidationError({
                f'{where}': _("Number must be a valid phone number written in E164 format.")
            })

    @staticmethod
    def check_start_end(start:int, end:int) -> None:
        if start is None or end is None:
            return
        if len(str(start))!=len(str(end)):
            raise ValidationError({
                'end': _("End number must be the same length as start number.")
            })
        if start > end:
            raise ValidationError({
                'end': _("End number must be greater than or equal to the start number.")
            })


class PhoneMaintainerValidator:

    @staticmethod
    def check_address(status, latitude, longitude, physical_address, crl) -> None:

        if status in ['active',]:

            #_________________
            # Physical Address
            if physical_address is None or physical_address.strip() == "":
                crl.append(CheckResult(LogLevelChoices.LOG_FAILURE, None, f'"{status}" Maintainers mandates an physical_address.', "physical_physical_address"))
            if not re.match("^([^\n]* \r?\n)+[^\n]+$", physical_address):
                crl.append(CheckResult(LogLevelChoices.LOG_FAILURE, None, f'"{status}" Maintainers physical_address must be multiline and each line must end with a space.', "physical_physical_address"))

            #___________________________
            # Latitude / Longitude (GPS)
            if latitude is None or latitude == 0:
                crl.append(CheckResult(LogLevelChoices.LOG_FAILURE, None, f'"{status}" Maintainers mandates latitude.', "latitude"))
            if longitude is None or longitude == 0:
                crl.append(CheckResult(LogLevelChoices.LOG_FAILURE, None, f'"{status}" Maintainers mandates longitude.', "longitude"))

        elif status in ['retired',]:

            #_________________
            # Physical physical_address
            if physical_address is not None or physical_address.strip() != "":
                crl.append(CheckResult(LogLevelChoices.LOG_FAILURE, None, f'"{status}" Maintainers forbids an physical_address.', "physical_physical_address"))

            #___________________________
            # Latitude / Longitude (GPS)
            if latitude is not None:
                crl.append(CheckResult(LogLevelChoices.LOG_FAILURE, None, f'"{status}" Maintainers forbids latitude.', "latitude"))
            if longitude is not None:
                crl.append(CheckResult(LogLevelChoices.LOG_FAILURE, None, f'"{status}" Maintainers forbids longitude.', "longitude"))


    @staticmethod
    def check_time_zone(status, time_zone, crl) -> None:
        if status not in ['retired',]:

            if time_zone is None:
                crl.append(CheckResult(LogLevelChoices.LOG_FAILURE, None, f'This maintainer if missing a timezone definition.', "time_zone"))


def number_quicksearch(start: int, end: int, pattern: str) -> bool:
    '''
    Recherche rapide d'un nombre dans une plage donnée
    '''
    pattern_len = len(pattern)
    pattern_int = int(pattern)
    divisor = 10 ** pattern_len

    if start % divisor == pattern_int or end % divisor == pattern_int:
        return True

    current = start
    while current <= end:
        temp = current
        while temp > 0:
            if temp % divisor == pattern_int:
                return True
            temp //= 10
        current += 1

    return False

